#include "model.h"

void ModelnData::impute_qc(int comp_mode)
{
	int ni = 0; 
	for (int i = 0; i < nIndiv; i++)
	{
		if(pIndiv[i]->get_mask() == 1) continue; 
		ni++; 
	}

	double ** v11 = Allocate2DMatrix(ni, nLoci); 
	double ** v12 = Allocate2DMatrix(ni, nLoci); 

	for (int i = 0; i < ni; i++)
	for (int m = 0; m < nLoci; m++)
		v11[i][m] = v12[i][m] = 0; 

	for (int e = 0; e < nEMRuns; e++)
	{
		double ** th = (pMP+e)->Gettheta(); 
		int ii = 0; 
		for (int i = 0; i < nIndiv; i++)
		{
			if(pIndiv[i]->get_mask() == 1) continue; 
			pIndiv[i]->compute_pk(nS, nK, (pMP+e), comp_mode); 
			for (int m = 0; m < nLoci; m++)
				v12[ii][m] += pIndiv[i]->GetsnpGT(m); 
			double ** pk = pIndiv[i]->get_pk(); 
			for (int m = 0; m < nLoci; m++)
				for (int k = 0; k < nK; k++)
					 v11[ii][m] += pk[m][k] * th[m][k]; 
			ii++; 
			pIndiv[i]->clean_after_em(1); 
		}
	}

	for (int i = 0; i < ni; i++)
	for (int m = 0; m < nLoci; m++)
	{
		v11[i][m] /= nEMRuns; 
		v12[i][m] /= nEMRuns; 
	}

	fstream fo; 
	string fn("output/");
	fn.append(fnOutput);
	fn.append(".qc.txt");

	fo.open(fn.c_str(), ios::out);
	if(!fo.good()) 
	{
		fplog << "warning: failed to open file to write." << endl;
		cout << "-warning: failed to open file to write" << endl;
		return; 
	}

	fo << " ##typed imputed" << endl; 
	for (int i = 0; i < ni; i++)
	{
		for (int m = 0; m < nLoci; m++)
		{
			char buf[100]; 
			sprintf(buf, " %5.3f", v11[i][m]); 
			fo << buf; 
		}
		fo << endl; 
	}
	for (int i = 0; i < ni; i++)
	{
		for (int m = 0; m < nLoci; m++)
		{
			char buf[100]; 
			sprintf(buf, " %5.3f", v12[i][m]); 
			fo << buf; 
		}
		fo << endl; 
	}
	fo.close(); 

	Free2DMatrix(v11); 
	Free2DMatrix(v12); 
}

void ModelnData::detect_error(int comp_mode)
{
	int ni = 0; 
	for (int i = 0; i < nIndiv; i++)
	{
		if(pIndiv[i]->get_mask() == 1) continue; 
		ni++; 
	}
	double ** diff = Allocate2DMatrix(ni, nLoci); 
	for (int i = 0; i < ni; i++)
		for (int m = 0; m < nLoci; m++)
			diff[i][m] = 0; 

	cout << endl << " num loci = " << nLoci << endl; 
	int unit = 1000; 
	if(nLoci > unit) 
	{ 
		int nfile = ceil(nLoci / (double) unit); 
		vector<int> pbreak; 
		for (int i = 0; i < nfile; i++)
			pbreak.push_back(i * unit); 
		pbreak.push_back(nLoci); 
		cout << nfile << " --- "; 
		for (unsigned i = 0; i < pbreak.size(); i++)
			cout << pbreak.at(i) << " " ; 
		cout << endl; 
		//bpreak has 11 elements; 

		double ** mpk = Allocate2DMatrix(ni, unit*nK); 
		for (int e = 0; e < nEMRuns; e++)
		{
			double ** theta = (pMP+e)->Gettheta(); 
			for (int j = 0; j < nfile; j++)
			{
				cout << j << " of " << nfile << " many blocks" << endl; 
				int ii = 0; 
				for (int i = 0; i < nIndiv; i++)
				{
					if(pIndiv[i]->get_mask() == 1) continue; 
					pIndiv[i]->compute_pk(nS, nK, (pMP+e), comp_mode, j, pbreak); 
					double ** pk = pIndiv[i]->get_pk(); 
   					for (int s = 0; s < pbreak[j+1] - pbreak[j]; s++)
						for (int k = 0; k < nK; k++)
							mpk[ii][s*nK+k] = pk[s][k]; 
					ii++;
					pIndiv[i]->clean_after_em(1); 
				}

				for (int s = 0; s < pbreak[j+1] - pbreak[j]; s++)
				{
					int m = s + pbreak[j]; 
					int ii = 0;
					for (int i = 0; i < nIndiv; i++)
					{
						if(pIndiv[i]->get_mask() == 1) continue; 
//						double temp = pIndiv[i]->GetsnpGT(m);  
						double impute = 0; 
						for (int k = 0; k < nK; k++)
							impute += mpk[ii][s*nK+k] * theta[m][k]; 
//						if(temp > 3) 
//							diff[ii][m] += 0; //impute; 
//						else 
//							diff[ii][m] += impute - temp; 
						diff[ii][m] += impute; 
						ii++;
					}
				}
			}

		}
		Free2DMatrix(mpk); 
		vector<int> ().swap(pbreak); 
	}
	else 
	{
		for (int e = 0; e < nEMRuns; e++)
		{
			double ** theta = (pMP+e)->Gettheta(); 
			for (int i = 0; i < nIndiv; i++)
			{
				if(pIndiv[i]->get_mask() == 1) continue; 
				pIndiv[i]->compute_pk(nS, nK, (pMP+e), comp_mode); 
				pIndiv[i]->clean_after_em(0); 
			}
			for (int m = 0; m < nLoci; m++)
			{
				int ii = 0; 
				for (int i = 0; i < nIndiv; i++)
				{
					if(pIndiv[i]->get_mask() == 1) continue; 
//					double temp = pIndiv[i]->GetsnpGT(m);  
					double ** pk = pIndiv[i]->get_pk(); 
					double impute = 0; 
					for (int k = 0; k < nK; k++)
						impute += pk[m][k] * theta[m][k]; 
//					if(temp > 3) 
//						diff[ii][m] += 0; //impute; 
//					else 
//						diff[ii][m] += impute - temp; 
   					diff[ii][m] += impute; 
					ii++;
				}
			}
		}
	}

	fstream fo; 
	string fn("output/");
	fn.append(fnOutput);
	fn.append(".diff.txt");

	fo.open(fn.c_str(), ios::out);
	if(!fo.good()) 
	{
		fplog << "ERROR: failed to open file to write." << endl;
		cout << "-warning: failed to open file to write" << endl;
		return; 
	}

	for (int m = 0; m < nLoci; m++)
	{
		for (int i = 0; i < ni; i++)
		{
			char buf[100]; 
			sprintf(buf, "%5.3f ", diff[i][m] /= nEMRuns); 
			fo << buf; 
		}
		fo << endl; 
	}
    fo.close(); 
}

void ModelnData::test_hpass_mgt(int comp_mode)
{
    vector<double> tph;
	std::fstream in;
	in.open(fnFILE.c_str());
	if(in.is_open()) {
		double temp = 0; 
		while (in >> temp) {
		  	tph.push_back(temp);
		}
		in.close();
	}
	else std::cout << "unable to open file" << std::endl;
 
	int ni = (int) tph.size(); 
	double * ph = new double[ni];
    for (int i = 0; i < ni; i++) 
		ph[i]=tph.at(i);
	vector<double> ().swap(tph); 
	

	double ** cov = NULL; 
	int nc = 0; 
	{
		in.open(fnDOC.c_str());
		if(in.is_open() == 1) {
			vector<vector<double> > b;
			std::istringstream iss;
			std::string buf;
			vector<double> tmpvec;
			while(getline(in, buf)){
			   iss.str(buf);
			   double tmp;
			   while(iss>>tmp){
				   tmpvec.push_back(tmp);
			   }
			   b.push_back(tmpvec);
			   tmpvec.clear() ;
			   iss.clear();
			}
			in.close();
			if((int) b.size() == ni) {
				nc = b.at(0).size(); 
				cov = Allocate2DMatrix(ni, nc);  
				for (int i = 0; i < ni; i++)
					for (int j = 0; j < nc; j++)
						cov[i][j] = b.at(i).at(j); 
			}
			else if(b.size() > 0)
			{
				cout << " dimension not match in cov !";
				fplog << " ###### dimension not match in cov !";
				exit(0); 
			}
			vector<vector<double> > ().swap(b); 
		}
	}

	int nLoci = 0; 
	double ** egt = NULL; 
	{
		string fn; 
		fstream fi; 

//		fn.append("output/");
//		fn.append(fnOutput);
//		fn.append(".try1.exact.genotype.012");
		fn.append(vGin.at(0)); 
		fi.open(fn.c_str(), ios::in);
		cout << fn << endl; 

		if(fi.is_open() == 1) {
		    std::istringstream iss;

		   	std::string buf;
		   	getline(fi, buf); 
		   	iss.str(buf);
		   	double tmp;
		   	int jj = 0; 
		   	while(iss>>tmp){
			    jj++; 
		   	}
		   nLoci = jj; 
		   iss.clear(); 
		   egt = Allocate2DMatrix(ni, nLoci); 
		   iss.str(buf); 
		   for (int m = 0; m < nLoci; m++)
		   {
		   	   iss >> egt[0][m]; 
		   }

			int ii = 1; 
			while(getline(fi, buf)){
			   iss.str(buf);
			   for (int m = 0; m < nLoci; m++)
			   {
				   iss >> egt[ii][m]; 
			   }
			   iss.clear();
			   ii++;
			}
			fi.close();
		}
		else {
			cout << "can't open file" << endl; 
			exit(0); 
		}
	}

	cout << ni << endl; 
	cout << nLoci << endl; 

	double ** igt = Allocate2DMatrix(ni, nLoci); 
	for (int i = 0; i < ni; i++)
		for (int m = 0; m < nLoci; m++)
			igt[i][m] = 0; 
//	memset((void*) &igt[0][0], 0, ni * nLoci * (size_t) sizeof(double)); 
	for (unsigned t = 0; t < vPin.size(); t++) 
	{
		string fn; 
		fstream fi; 
//		char buf[100]; 
//		sprintf(buf, ".try%d.combine.imputed.txt", t); 
//
//		fn.append("output/");
//		fn.append(fnOutput);
//		fn.append(buf); 
		fn.append(vPin.at(t)); 
		fi.open(fn.c_str(), ios::in);

		cout << fn << endl; 
		if(fi.is_open() == 1) {
			std::istringstream iss;
			std::string buf;
			double tmp; 

			int ii = 0; 
			while(getline(fi, buf)){
			   iss.str(buf);
			   for (int m = 0; m < nLoci; m++)
			   {
			       iss >> tmp; 
				   igt[ii][m] += tmp; 
			   }
			   iss.clear();
			   ii++; 
			}
			fi.close();
		}
	}
	for (int i = 0; i < ni; i++)
		for (int m = 0; m < nLoci; m++)
			igt[i][m] /= vPin.size(); 
    
	double * bf1 = new double[nLoci];
	double * lrt1 = new double[nLoci];
	double * bf2 = new double[nLoci]; 
	double * lrt2 = new double[nLoci]; 
	double * bflhs = new double[nLoci]; 
	double * lrtlhs = new double[nLoci]; 


	double * beta = new double[nK+1+nc]; 
	double ** mgt = Allocate2DMatrix(ni, 2); 
	double ** temp = Allocate2DMatrix(ni, 1); 

	for (int m = 0; m < nLoci; m++)
	{
		for (int i = 0; i < ni; i++)
			mgt[i][0] = egt[i][m]; 
	    bf1[m] = bf_linear(ni, 1, ph, mgt, beta, nc, cov);  
	    lrt1[m] = lrt_logistic(ni, 1, ph, mgt, beta, nc, cov); 
		if(lrt1[m] < 0) lrt1[m] = 0; 

//		for (int s = 0; s < nc+2; s++)
//		{
//			char buf[100]; 
//			sprintf(buf, "%5.3f ", beta[s]); 
//			fplog << buf; 
//		}
//		fplog << endl; 

		for (int i = 0; i < ni; i++)
			mgt[i][0] = igt[i][m]; 
		bflhs[m] = bf_linear(ni, 1, ph, mgt, beta, nc, cov);  
		lrtlhs[m] = lrt_logistic(ni, 1, ph, mgt, beta, nc, cov);  
		if(lrtlhs[m] < 0) lrtlhs[m] = 0; 

		for (int i = 0; i < ni; i++)
			mgt[i][1] = egt[i][m]; 
		bf2[m] = bf_linear(ni, 2, ph, mgt, beta, nc, cov);  
		lrt2[m] = lrt_logistic(ni, 2, ph, mgt, beta, nc, cov); 
		if(lrt2[m] < 0) lrt2[m] = 0; 

		//exact genotype must be at the last position; 
//		for (int s = 0; s < nc+3; s++)
//		{
//			char buf[100]; 
//			sprintf(buf, "%5.3f ", beta[s]); 
//			fplog << buf; 
//		}
//		fplog << endl; 
	}


	fstream fo; 
	string fn("output/");
	fn.append(fnOutput);
	fn.append(".combine.bf.txt");

	fo.open(fn.c_str(), ios::out);
	if(!fo.good()) 
	{
		fplog << "ERROR: failed to open file to write." << endl;
		cout << "-warning: failed to open file to write" << endl;
		return; 
	}

	vector<int> index; 
	fo << "lrt2\t bf2\t lrtlhs\t bflhs\t lrt1\t bf1" << endl; 
	for (int m = 0; m < nLoci; m++)
	{
		char buf[100]; 
		double p2 = -log10(gsl_cdf_chisq_Q(lrt2[m], 2));
		if(p2 > 300) p2 = 300; 
		double p1 = -log10(gsl_cdf_chisq_Q(lrt1[m], 1));
		if(p1 > 300) p1 = 300; 
		double plhs =  -log10(gsl_cdf_chisq_Q(lrtlhs[m], 1)); 
//		double pc =  -log10(gsl_cdf_chisq_Q(lrtc[m], 1)); 

		sprintf(buf, "%5.3f \t %5.3f \t %5.3f \t %5.3f \t %5.3f \t %5.3f\n", p2, bf2[m]/log(10.0), plhs, bflhs[m]/log(10.0), p1, bf1[m]/log(10.0)); 
		fo << buf; 
//		if(p2 > 6 || p1 > 6) 
			index.push_back(m); 
	}
	fo.close(); 

	//write the combined imputed genotypes; 
	{
		string fn; 
		fstream outfile; 

		fn.append("output/");
		fn.append(fnOutput);
		fn.append(".combine.imputed.txt");
//		fn.append(".lg6.txt"); 
//		fn.append(".lg6.txt"); 
		outfile.open(fn.c_str(), ios::out);
		if(!outfile.good()) 
		{
			fplog << "ERROR: failed to open file to write." << endl;
			cout << "-warning: failed to open file to write" << endl;
			return; 
		}
		for (int i = 0; i < ni; i++)
		{
			char buf[100]; 
//			for (int m = 0; m < nLoci; m++)
			for (unsigned j = 0; j < index.size(); j++)
			{
                int m = index.at(j); 
				sprintf(buf, "%5.3f ", egt[i][m]); 
				outfile << buf; 
			}
			outfile << endl; 
		}
		for (int i = 0; i < ni; i++)
		{
			char buf[100]; 
//			for (int m = 0; m < nLoci; m++)
			for (unsigned j = 0; j < index.size(); j++)
			{
                int m = index.at(j); 
				sprintf(buf, "%5.3f ", igt[i][m]); 
				outfile << buf; 
			}
			outfile << endl; 
		}
		outfile.close(); 
	}

	Free2DMatrix(egt); 
	Free2DMatrix(igt); 
	Free2DMatrix(mgt); 
	Free2DMatrix(cov); 
	Free2DMatrix(temp); 
	delete[] ph; 
	delete[] beta; 
	delete[] bf2; 
	delete[] lrt2; 
	delete[] bf1; 
	delete[] lrt1; 
	delete[] bflhs; 
	delete[] lrtlhs; 
}

void ModelnData::selection(int comp_mode)
{
	double * v11 = new double[nLoci]; 
	double * v22 = new double[nLoci]; 
	double * v33 = new double[nLoci]; 
	double * v12 = new double[nLoci]; 
	double * v13 = new double[nLoci]; 
	double * v23 = new double[nLoci]; 
	for (int m = 0; m < nLoci; m++)
		v11[m] = v22[m] = v33[m] = v12[m] = v13[m] = v23[m] = 0; 

	int ni = 0; 
	for (int i = 0; i < nIndiv; i++)
	{
		if(pIndiv[i]->get_mask() == 1) continue; 
		ni++; 
	}

	vector<int> set1; 
	vector<int> set2; 
	vector<int> set3; 
	int ii = 0; 
	for (int i = 0; i < nIndiv; i++)
	{
		if(pIndiv[i]->get_mask() == 1) continue; 
		if(m_phval.at(i) == 1) set1.push_back(ii); 
		else if(m_phval.at(i) == 2) set2.push_back(ii); 
		else set3.push_back(ii); 
		ii++; 
	}

	cout << endl << " num loci = " << nLoci << endl; 
	int unit = 1000; 
	if(nLoci > unit) 
	{ 
		int nfile = ceil(nLoci / (double) unit); 
		vector<int> pbreak; 
		for (int i = 0; i < nfile; i++)
			pbreak.push_back(i * unit); 
		pbreak.push_back(nLoci); 
		cout << nfile << " --- "; 
		for (unsigned i = 0; i < pbreak.size(); i++)
			cout << pbreak.at(i) << " " ; 
		cout << endl; 
		//bpreak has 11 elements; 

		double ** mpk = Allocate2DMatrix(ni, unit*nK); 
		for (int e = 0; e < nEMRuns; e++)
		{
			for (int j = 0; j < nfile; j++)
			{
				cout << j << " of " << nfile << " many blocks" << endl; 
				int ii = 0; 
				for (int i = 0; i < nIndiv; i++)
				{
					if(pIndiv[i]->get_mask() == 1) continue; 
					pIndiv[i]->compute_pk(nS, nK, (pMP+e), comp_mode, j, pbreak); 
					double ** pk = pIndiv[i]->get_pk(); 
   					for (int s = 0; s < pbreak[j+1] - pbreak[j]; s++)
						for (int k = 0; k < nK; k++)
							mpk[ii][s*nK+k] = pk[s][k]; 
					ii++;
					pIndiv[i]->clean_after_em(1); 
				}

				for (int s = 0; s < pbreak[j+1] - pbreak[j]; s++)
				{
					int m = s + pbreak[j]; 
					
					for (int k = 0; k < nK; k++)
					{
						double sum1 = 0; 
						double sum2 = 0; 
						for (unsigned i1 = 0; i1 < set1.size(); i1++)
						{
                             sum1 += mpk[set1.at(i1)][s*nK+k]; 
							 sum2 += mpk[set1.at(i1)][s*nK+k] * mpk[set1.at(i1)][s*nK+k]; 
						}
						v11[m] += sum1 * sum1;  
						v12[m] += sum2 ;  
					}

                    if(set2.size() > 0) {
						for (int k = 0; k < nK; k++)
						{
							double sum1 = 0; 
							double sum2 = 0; 
							for (unsigned i1 = 0; i1 < set2.size(); i1++)
							{
								 sum1 += mpk[set2.at(i1)][s*nK+k]; 
								 sum2 += mpk[set2.at(i1)][s*nK+k] * mpk[set2.at(i1)][s*nK+k]; 
							}
							v22[m] += sum1 * sum1;  
							v13[m] += sum2 ;  
						}
					}

					if(set3.size() > 0) {
						for (int k = 0; k < nK; k++)
						{
							double sum1 = 0; 
							double sum2 = 0; 
							for (unsigned i1 = 0; i1 < set3.size(); i1++)
							{
								 sum1 += mpk[set3.at(i1)][s*nK+k]; 
								 sum2 += mpk[set3.at(i1)][s*nK+k] * mpk[set3.at(i1)][s*nK+k]; 
							}
							v33[m] += sum1 * sum1;  
							v23[m] += sum2 ;  
						}
					}
				}
			}
		}
		Free2DMatrix(mpk); 
		vector<int> ().swap(pbreak); 
	}
	else 
	{

		double ** mpk = Allocate2DMatrix(ni, nLoci*nK); 
		for (int e = 0; e < nEMRuns; e++)
		{
			int ii = 0; 
			for (int i = 0; i < nIndiv; i++)
			{
				if(pIndiv[i]->get_mask() == 1) continue; 
				pIndiv[i]->compute_pk(nS, nK, (pMP+e), comp_mode); 
				double ** pk = pIndiv[i]->get_pk(); 
				for (int m = 0; m < nLoci; m++)
					for (int k = 0; k < nK; k++)
						mpk[ii][m*nK+k] = pk[m][k]; 
				ii++; 
				pIndiv[i]->clean_after_em(1); 
			}
			for (int m = 0; m < nLoci; m++)
			{
				for (int k = 0; k < nK; k++)
				{
					double sum1 = 0; 
					double sum2 = 0; 
					for (unsigned i1 = 0; i1 < set1.size(); i1++)
					{
						 sum1 += mpk[set1.at(i1)][m*nK+k]; 
						 sum2 += mpk[set1.at(i1)][m*nK+k] * mpk[set1.at(i1)][m*nK+k]; 
					}
					v11[m] += sum1 * sum1;  
					v12[m] += sum2 ;  
				}

				if(set2.size() > 0) {
					for (int k = 0; k < nK; k++)
					{
						double sum1 = 0; 
						double sum2 = 0; 
						for (unsigned i1 = 0; i1 < set2.size(); i1++)
						{
							 sum1 += mpk[set2.at(i1)][m*nK+k]; 
							 sum2 += mpk[set2.at(i1)][m*nK+k] * mpk[set2.at(i1)][m*nK+k]; 
						}
						v22[m] += sum1 * sum1;  
						v13[m] += sum2 ;  
					}
				}

				if(set3.size() > 0) {
					for (int k = 0; k < nK; k++)
					{
						double sum1 = 0; 
						double sum2 = 0; 
						for (unsigned i1 = 0; i1 < set3.size(); i1++)
						{
							 sum1 += mpk[set3.at(i1)][m*nK+k]; 
							 sum2 += mpk[set3.at(i1)][m*nK+k] * mpk[set3.at(i1)][m*nK+k]; 
						}
						v33[m] += sum1 * sum1;  
						v23[m] += sum2 ;  
					}
				}
			}
		}

		Free2DMatrix(mpk); 
	}
	
	for (int m = 0; m < nLoci; m++)
	{
		int n1 = set1.size(); 
		v11[m] -= v12[m]; 
		v11[m] /= (n1 * (n1-1) * nEMRuns); 
		int n2 = set2.size(); 
		if(n2 > 0) {
			v22[m] -= v13[m]; 
			v22[m] /= (n2 * (n2-1) * nEMRuns); 
		}
		int n3 = set3.size(); 
		if(n3 > 0) {
			v33[m] -= v23[m]; 
			v33[m] /= (n3 * (n3-1) * nEMRuns); 
		}
	}

	fstream fo; 
	string fn("output/");
	fn.append(fnOutput);
	fn.append(".sel2.txt");

	fo.open(fn.c_str(), ios::out);
	if(!fo.good()) 
	{
		fplog << "ERROR: failed to open file to write." << endl;
		cout << "-warning: failed to open file to write" << endl;
		return; 
	}

	fo << " ##v11, v22, v33" << endl; 
	for (int m = 0; m < nLoci; m++)
	{
		char buf[100]; 
		sprintf(buf, " %5.3f \t %5.3f \t %5.3f\n", v11[m], v22[m], v33[m]); 
		fo << buf; 
	}
	fo.close(); 

	delete[] v11; 
	delete[] v22; 
	delete[] v33; 
	delete[] v12; 
	delete[] v13; 
	delete[] v23; 
}

void ModelnData::compute_lhs(int comp_mode)  //this compute pairwise lhs. 
{
	int ni = 0; 
	for (int i = 0; i < nIndiv; i++)
	{
		if(pIndiv[i]->GetisPanel() == 1 || pIndiv[i]->get_mask() == 1) continue; 
		ni++; 
	}

	double ** lhs = Allocate2DMatrix(ni, ni); 
	for (int i = 0; i < ni; i++)
		for (int j = 0; j < ni; j++)
			lhs[i][j] = 0; 

	int unit = 1000; 
	int nfile = ceil(nLoci / (double) unit); 
	vector<int> pbreak; 
	for (int i = 0; i < nfile; i++)
		pbreak.push_back(i * unit); 
	pbreak.push_back(nLoci); 
	cout << nfile << " --- "; 
	for (unsigned i = 0; i < pbreak.size(); i++)
		cout << pbreak.at(i) << " " ; 
	cout << endl; 

	double ** gt = Allocate2DMatrix(ni, unit*nK); 
	for (int e = 0; e < nEMRuns; e++)
	{
		for (int f = 0; f < nfile; f++)
		{
			cout << f << " of " << nfile << " many blocks" << endl; 
			int ii = 0; 
			for (int i = 0; i < nIndiv; i++)
			{
				if(pIndiv[i]->GetisPanel() == 1 || pIndiv[i]->get_mask() == 1) continue; 
				pIndiv[i]->compute_pk(nS, nK, (pMP+e), comp_mode, f, pbreak); 
				double ** pk = pIndiv[i]->get_pk(); 
				for (int s = 0; s < pbreak[f+1] - pbreak[f]; s++)
				{
					for (int k = 0; k < nK; k++)
						gt[ii][s*nK+k] = pk[s][k]; 
				}
				ii++;
				pIndiv[i]->clean_after_em(1); 
			}

			for (int i = 0; i < ni; i++)
				for (int j = i; j < ni; j++)
				{
					for (int s = 0; s < pbreak[f+1] - pbreak[f]; s++) 
						for (int k = 0; k < nK; k++)
							lhs[i][j] += gt[i][s*nK+k] * gt[j][s*nK+k]; 
				}
		}

	}
	Free2DMatrix(gt); 
	vector<int> ().swap(pbreak); 

	for (int i = 0; i < ni; i++)
		for (int j = i; j < ni; j++)
			lhs[i][j] /= (double) nEMRuns;   

	fstream fo; 
	string fn("output/");
	fn.append(fnOutput);
	fn.append(".lhs.txt");

	fo.open(fn.c_str(), ios::out);
	if(!fo.good()) 
	{
		fplog << "ERROR: failed to open file to write." << endl;
		cout << "-warning: failed to open file to write" << endl;
		return; 
	}
    char buf[100]; 
	for (int i = 0; i < ni; i++)
	{
		for (int j = 0; j < ni; j++)
		{
			double temp = 0; 
			if(j < i) temp = lhs[j][i]; 
			else temp = lhs[i][j]; 
			sprintf(buf, "%5.3f ", temp); 
			fo << buf; 
		}
		fo << endl; 
	}

	fo.close(); 

	Free2DMatrix(lhs); 
}

void ModelnData::combine_lhs(int comp_mode)
{
	int ni = m_num; 
	double ** lhs = Allocate2DMatrix(ni, ni); ; 
	for (int i = 0; i < ni; i++)
		for (int j = 0; j < ni; j++)
			lhs[i][j] = 0; 

	for (unsigned t = 0; t < vGin.size(); t++) 
	{
		string fn; 
		fstream fi; 
		fn.append(vGin.at(t)); 
		fi.open(fn.c_str(), ios::in);

		cout << fn << endl; 
		if(fi.is_open() == 1) {
			std::istringstream iss;
			std::string buf;
			double tmp; 

			int ii = 0; 
			while(getline(fi, buf)){
			   iss.str(buf);
			   for (int m = 0; m < ni; m++)
			   {
			       iss >> tmp; 
				   lhs[ii][m] += tmp; 
			   }
			   iss.clear();
			   ii++; 
			}
			if(ii != ni) { 
				cout << "number of row doesn't match!" << endl; 
				exit(0); 
			}
			fi.close();
		}
	}

	fstream fo; 
	string fn("output/");
	fn.append(fnOutput);
	fn.append(".lhs+.txt");

	fo.open(fn.c_str(), ios::out);
	if(!fo.good()) 
	{
		fplog << "ERROR: failed to open file to write." << endl;
		cout << "-warning: failed to open file to write" << endl;
		return; 
	}
    char buf[100]; 
	for (int i = 0; i < ni; i++)
	{
		for (int j = 0; j < ni; j++)
		{
			double temp = 0; 
			if(j < i) temp = lhs[j][i]; 
			else temp = lhs[i][j]; 
			sprintf(buf, "%5.3f ", temp); 
			fo << buf; 
		}
		fo << endl; 
	}

	fo.close(); 

	Free2DMatrix(lhs); 
}
